{ Welcome to the wonderful world of Zlib

  zlib 1.1.4 is a general purpose data compression library.
  All the code is thread safe.  The data format used by the
  zlib library is described by RFCs (Request for Comments)
  1950 to 1952 in the files http://www.ietf.org/rfc/rfc1950.txt
  (zlib format), rfc1951.txt (deflate format) and rfc1952.txt
  (gzip format). These documents are also available in other
  formats from
    ftp://ftp.uu.net/graphics/png/documents/zlib/zdoc-index.html

  This unit is based on zlib 1.1.4
  
  Differences to zlib.h:
  ======================
  - No underscores
  - ZLIB_VERSION is renamed to ZLibVersionConst
  - function ZlibVersion returns String(255)
  - function GzEof returns boolean value
  - GzError returns String(255)
  - SEEK_SET and SEEK_CUR are introduced in this unit
    as SeekSet and SeekCur. These consts are needed as
    the "whence" parameter to GzSeek
  
  Copyright (C) 2002 by Eike Lange

  Author: Eike Lange <eike@g-n-u.de>

  This unit is free software; you can redistribute it and/or modify
  it under the terms of the GNU Library General Public License as
  published by the Free Software Foundation, version 2.

  This unit is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
  Library General Public License for more details.

  You should have received a copy of the GNU Library General Public
  License along with this library; see the file COPYING.LIB. If not,
  write to the Free Software Foundation, Inc., 59 Temple Place -
  Suite 330, Boston, MA 02111-1307, USA. }

{ ORIGINAL COPYRIGHT of zlib.h 
  ============================
  zlib.h -- interface of the 'zlib' general purpose compression library
  version 1.1.4, March 11th, 2002

  Copyright (C) 1995-2002 Jean-loup Gailly and Mark Adler

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The origin of this software must not be misrepresented; you must not
     claim that you wrote the original software. If you use this software
     in a product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

  Jean-loup Gailly        Mark Adler
  jloup@gzip.org          madler@alumni.caltech.edu


  The data format used by the zlib library is described by RFCs (Request for
  Comments) 1950 to 1952 in the files ftp://ds.internic.net/rfc/rfc1950.txt
  (zlib format), rfc1951.txt (deflate format) and rfc1952.txt (gzip format).
}

{$gnu-pascal,I-}

unit ZLib;

interface

  {@section zlib - types }
type
  PByteF    = ^ByteF;
  PuLongF   = ^Cardinal;
  PZStream  = ^ZStream;
  ByteF     = Byte;
  uInt      = Cardinal;
  uLong     = MedCard;
  GzFile    = Pointer;

  String255 = String(255);

  TAllocFunc = ^function (opaque: Pointer; Items, aSize: uInt): Pointer;
  TFreeFunc  = ^procedure (opaque, address: Pointer);

  ZStream = record
    NextIn:   PByteF;
    AvailIn:  uInt;
    TotalIn:  uLong;
    NextOut:  PByteF;
    AvailOut: uInt;
    TotalOut: uLong;
    Msg:      PChar;
    State:    Pointer;
    ZAlloc:   TAllocFunc;
    ZFree:    TFreeFunc;
    opaque:   Pointer;
    DataType: Integer;
    Adler, Reserved: uLong
  end;

var
  { PLEASE DO NOT ASSIGN ANYTHING TO THESE IDENTIFIERS !!!
    Treat them as const.
  =======================================================}

  {@section consts}
  {!! Different Name}
  ZLibVersionConst:   PChar;    asmname 'czlib_version'; external;
  ZNoFlush:           Integer;  asmname 'cz_no_flush'; external;
  ZPartialFlush:      Integer;  asmname 'cz_partial_flush'; external;
  ZSyncFlush:         Integer;  asmname 'cz_sync_flush'; external;
  ZFullFlush:         Integer;  asmname 'cz_full_flush'; external;
  ZFinish:            Integer;  asmname 'cz_finish'; external;
  ZOk:                Integer;  asmname 'z_ok'; external;
  ZStreamEnd:         Integer;  asmname 'cz_stream_end'; external;
  ZNeedDict:          Integer;  asmname 'cz_need_dict'; external;
  ZErrno:             Integer;  asmname 'cz_errno'; external;
  ZStreamError:       Integer;  asmname 'cz_stream_error'; external;
  ZDataError:         Integer;  asmname 'cz_data_error'; external;
  ZMemError:          Integer;  asmname 'cz_mem_error'; external;
  ZBufError:          Integer;  asmname 'cz_buf_error'; external;
  ZVersionError:      Integer;  asmname 'cz_version_error'; external;
  ZNoCompression:     Integer;  asmname 'cz_no_compression'; external;
  ZBestSpeed:         Integer;  asmname 'cz_best_speed'; external;
  ZBestCompression:   Integer;  asmname 'cz_best_compression'; external;
  ZDefaultCompression:Integer;  asmname 'cz_default_compression'; external;
  ZFiltered:          Integer;  asmname 'cz_filtered'; external;
  ZHuffmanOnly:       Integer;  asmname 'cz_huffman_only'; external;
  ZDefaultStrategy:   Integer;  asmname 'cz_default_strategy'; external;
  ZBinary:            Integer;  asmname 'cz_binary'; external;
  ZAscii:             Integer;  asmname 'cz_ascii'; external;
  ZUnknown:           Integer;  asmname 'cz_unknown'; external;
  ZDeflated:          Integer;  asmname 'cz_deflated'; external;
  ZNull:              Integer;  asmname 'cz_null'; external;
  { Whence for GzSeek }
  SeekSet:            Integer;  asmname 'cseek_set'; external;
  SeekCur:            Integer;  asmname 'cseek_current'; external;


  {@section Prototypes}
  {@subsection Basic Functions }


{!! Different from C }
function ZlibVersion: String255;

function DeflateInit (Stream: PZStream; Level: Integer): Integer;
asmname 'deflateInit';

function Deflate (Stream: PZStream; Flush: Integer): Integer;
asmname 'deflate';

function DeflateEnd (Stream: PZStream): Integer;
asmname 'deflateEnd';

function InflateInit (Stream: PZStream): Integer;
asmname 'inflateInit';

function Inflate (Stream: PZStream; Flush: Integer): Integer;
asmname 'inflate';

function InflateEnd (Stream: PZStream): Integer;
asmname 'inflateEnd';

function DeflateInit2 (Stream: PZStream; Level, Method, WindowBits,
  MemLevel, Strategy: Integer): Integer;
asmname 'deflateInit2';

function DeflateSetDictionary (Stream: PZStream; const Dictionary: PByteF;
  DictLength: uInt): Integer;
asmname 'deflateSetDictionary';

function DeflateCopy (Dest, Source: PZStream): Integer;
asmname 'deflateCopy';

function DeflateReset (Stream: PZStream): Integer;
asmname 'deflateReset';

function DeflateParams (Stream: PZStream; Level, Strategy: Integer): Integer;
asmname 'deflateParams';

function InflateInit2 (Stream: PZStream; WindowBits: Integer): Integer;
asmname 'inflateInit2';

function InflateSetDictionary (Stream: PZStream; const Dictionary: PByteF;
  DictLength: uInt): Integer;
asmname 'inflateSetDictionary';

function InflateSync (Stream: PZStream): Integer;
asmname 'inflateSync';

function InflateReset (Stream: PZStream): Integer;
asmname 'inflateReset';

  {@subsection Utility Functions }

function Compress (Dest: PByteF; DestLen: PuLongF;
  const Src: PByteF; SrcLen: uLong): Integer;
asmname 'compress';

function Compress2 (Dest: PByteF; DestLen: PuLongF;
  const Src: PByteF; SrcLen: uLong): Integer;
asmname 'compress2';

function Uncompress (Dest: PByteF; DestLen: PuLongF;
  const Src: PByteF; SrcLen: uLong): Integer;
asmname 'uncompress';

  {@subsection GZIP Functions }

function GzOpen (Path, Mode: PChar): GzFile;
asmname 'gzopen';

function GzDOpen (var f: File; Mode: PChar): GzFile;
asmname 'gzdopen';

function GzSetParams (f: GzFile; Level, Strategy: Integer): Integer;
asmname 'gzsetparams';

function GzRead (f: GzFile; Buffer: Pointer; Len: Cardinal): Integer;
asmname 'gzread';

function GzWrite (f: GzFile; Buffer: Pointer; Len: Cardinal): Integer;
asmname 'gzwrite';

function GzPrintF (f: GzFile; Format: PChar; ...): Integer;
asmname 'gzprintf';

function GzPutS (f: GzFile; s: PChar): Integer;
asmname 'gzputs';

function GzGetS (f: GzFile; Buffer: PChar; Len: Integer): Integer;
asmname 'gzgets';

function GzPutC (f: GzFile; c: Integer): Integer;
asmname 'gzputc';

function GzGetC (f: GzFile): Integer;
asmname 'gzgetc';

function GzFlush (f: GzFile; Flush: Integer): Integer;
asmname 'gzflush';

function GzSeek (f: GzFile; Offst: MedInt; Whence: Integer): MedInt;
asmname 'gzseek';

function GzRewind (f: GzFile): Integer;
asmname 'gzrewind';

function GzTell (f: GzFile): MedInt;
asmname 'gztell';

{!! different from C, returns TRUE, if EOF is detected }
function GzEof (f: GzFile): Boolean;

function GzClose (f: GzFile): Integer;
asmname 'gzclose';

{!! Different from C, returns String instead of PChar }
function GzError (f: GzFile; var ErrNum: Integer): String255;

  {@subsection Checksum Functions }

function Adler32 (Adler: uLong; const Buffer: PByteF; Len: uInt): uLong;
asmname 'adler32';

function CRC32 (crc: uLong; const Buffer: PByteF; Len: uInt): uLong;
asmname 'crc32';


implementation

{$L zlibc.c,z}

function CZlibVersion: PChar;
asmname 'zlibVersion';

function ZlibVersion: String255;
begin
  ZlibVersion := CString2String (CZlibVersion)
end;

function CGzEof (f: GzFile): Integer;
asmname 'gzeof';

function GzEof (f: GzFile): Boolean;
begin
  GzEof := (CGzEof (f) = 1)
end;

function CGzError (f: GzFile; var ErrNum: Integer): PChar;
asmname 'gzerror';

function GzError (f: GzFile; var ErrNum: Integer): String255;
begin
  GzError := CString2String (CGzError (f, ErrNum))
end;

{ The following functions are just to test the sizes of the above
record ZStream }
function CZStreamSize: Cardinal;
asmname 'cz_stream_size';

function PZStreamSize: Cardinal;
begin
  PZStreamSize := SizeOf (ZStream)
end;


to begin do
  begin
    if CZStreamSize <> PZStreamSize then
      begin
        Write (StdErr, 'ERROR: Sizes not the same. Do not use');
        WriteLn (StdErr, ' "zlib.pas" and contact `eike@g-n-u.de''.');
        Write (StdErr, 'ERROR: Please have a look at');
        WriteLn (StdErr, '`http://www.gnu-pascal.de/contrib/eike/''');
        Halt
      end
  end;
end.
